package jmunit.framework.cldc10;

import j2meunit.framework.Test;
import j2meunit.framework.TestCase;
import j2meunit.framework.TestSuite;
import jmunit.framework.cldc10.mocks.MockTestCase;
import jmunit.framework.cldc10.mocks.MockTestResult;

import com.hammingweight.hammock.Hammock;
import com.hammingweight.hammock.MethodHandler;

public class TestCaseTest extends TestCase {

	// The OUT.
	private MockTestCase testCase;

	// The mocks.
	private Hammock hammock;

	private MockTestResult mockResult;

	public void setUp() {
		// Create the OUT.
		this.testCase = AllTestsRunner.testCase;
		this.testCase.setInvocationHandler(new MethodHandler(
				MockTestCase.MTHD_COUNT_TEST_CASES)
				.setNumberInvocations(0, 100).setReturnValue(new Integer(3)));

		// Create the mocks.
		this.hammock = new Hammock();
		this.mockResult = new MockTestResult(this.hammock);
	}

	public void testCountTestCases() {
		assertEquals(3, this.testCase.countTestCases());
	}

	// We expect that the tests will be run and the TestResult object notified.
	public void testRunOk() {
		this.testCase.setInvocationHandler(this.hammock);
		// The TestResult object should be told that 3 tests started and that 3
		// tests ended.
		this.hammock.setExpectation(
				MockTestResult.MTHD_START_TEST_$_CLASS_STRING, new Object[] {
						this.testCase.getClass(), "testcase#0" });
		this.hammock.setExpectation(
				MockTestResult.MTHD_START_TEST_$_CLASS_STRING, new Object[] {
						this.testCase.getClass(), "testcase#1" });
		this.hammock.setExpectation(
				MockTestResult.MTHD_START_TEST_$_CLASS_STRING, new Object[] {
						this.testCase.getClass(), "testcase#2" });
		this.hammock
				.setExpectation(MockTestResult.MTHD_END_TEST_$_CLASS_STRING)
				.setNumberInvocations(3);
		// The setup, test and teardown methods should be called 3 times.
		this.hammock.setExpectation(MockTestCase.MTHD_SET_UP)
				.setNumberInvocations(3);
		this.hammock.setExpectation(MockTestCase.MTHD_TEST_$_INT,
				new Object[] { new Integer(0) });
		this.hammock.setExpectation(MockTestCase.MTHD_TEST_$_INT,
				new Object[] { new Integer(1) });
		this.hammock.setExpectation(MockTestCase.MTHD_TEST_$_INT,
				new Object[] { new Integer(2) });
		this.hammock.setExpectation(MockTestCase.MTHD_TEAR_DOWN)
				.setNumberInvocations(3);

		// Exercise.
		this.testCase.run(this.mockResult);

		// Verify.
		this.hammock.verify();
	}

	// Tests that if an exception is thrown in setUp that it's reported.
	public void testSetupException() {
		this.testCase.setInvocationHandler(this.hammock);
		// The TestResult object should be told that 3 tests started and that 3
		// tests ended.
		this.hammock.setExpectation(
				MockTestResult.MTHD_START_TEST_$_CLASS_STRING)
				.setNumberInvocations(3);
		this.hammock
				.setExpectation(MockTestResult.MTHD_END_TEST_$_CLASS_STRING)
				.setNumberInvocations(3);
		// The setup and teardown methods should be called 3 times.
		// The test method should only be called twice, not if setUp failed.
		RuntimeException re = new RuntimeException();
		this.hammock.setExpectation(MockTestCase.MTHD_SET_UP).setThrowable(re);
		this.hammock.setExpectation(MockTestCase.MTHD_SET_UP)
				.setNumberInvocations(2);
		this.hammock.setExpectation(MockTestCase.MTHD_TEST_$_INT)
				.setNumberInvocations(2);
		this.hammock.setExpectation(MockTestCase.MTHD_TEAR_DOWN)
				.setNumberInvocations(3);
		// The exception should be reported to the testResult.
		this.hammock.setExpectation(
				MockTestResult.MTHD_ADD_ERROR_$_CLASS_STRING_THROWABLE,
				new Object[] { this.testCase.getClass(), "testcase#0", re });

		// Exercise.
		this.testCase.run(this.mockResult);

		// Verify.
		this.hammock.verify();
	}

	// Tests that if an exception is thrown in tearDown that it's reported.
	public void testTearDownException() {
		this.testCase.setInvocationHandler(this.hammock);
		// The TestResult object should be told that 3 tests started and that 3
		// tests ended.
		this.hammock.setExpectation(
				MockTestResult.MTHD_START_TEST_$_CLASS_STRING)
				.setNumberInvocations(3);
		this.hammock
				.setExpectation(MockTestResult.MTHD_END_TEST_$_CLASS_STRING)
				.setNumberInvocations(3);
		// The setup, test and teardown methods should be called 3 times.
		RuntimeException re = new RuntimeException();
		this.hammock.setExpectation(MockTestCase.MTHD_SET_UP)
				.setNumberInvocations(3);
		this.hammock.setExpectation(MockTestCase.MTHD_TEST_$_INT)
				.setNumberInvocations(3);
		this.hammock.setExpectation(MockTestCase.MTHD_TEAR_DOWN).setThrowable(
				re);
		this.hammock.setExpectation(MockTestCase.MTHD_TEAR_DOWN)
				.setNumberInvocations(2);
		// The exception should be reported to the testResult.
		this.hammock.setExpectation(
				MockTestResult.MTHD_ADD_ERROR_$_CLASS_STRING_THROWABLE,
				new Object[] { this.testCase.getClass(), "testcase#0", re });

		// Exercise.
		this.testCase.run(this.mockResult);

		// Verify.
		this.hammock.verify();
	}

	// If an exception is thrown during setup and tearDown we should report both
	// exceptions.
	public void testSetupTearDownExceptions() {
		this.testCase.setInvocationHandler(this.hammock);
		// The TestResult object should be told that 3 tests started and that 3
		// tests ended.
		this.hammock.setExpectation(
				MockTestResult.MTHD_START_TEST_$_CLASS_STRING)
				.setNumberInvocations(3);
		this.hammock
				.setExpectation(MockTestResult.MTHD_END_TEST_$_CLASS_STRING)
				.setNumberInvocations(3);
		// The setup and teardown methods should be called 3 times.
		RuntimeException re1 = new RuntimeException();
		RuntimeException re2 = new RuntimeException();
		this.hammock.setExpectation(MockTestCase.MTHD_SET_UP)
				.setNumberInvocations(3).setThrowable(re1);
		this.hammock.setExpectation(MockTestCase.MTHD_TEAR_DOWN)
				.setNumberInvocations(3).setThrowable(re2);
		// The exceptions should be reported to the testResult.
		this.hammock.setExpectation(
				MockTestResult.MTHD_ADD_ERROR_$_CLASS_STRING_THROWABLE,
				new Object[] { this.testCase.getClass(), null, re1 })
				.ignoreArgument(1).setNumberInvocations(3);
		this.hammock.setExpectation(
				MockTestResult.MTHD_ADD_ERROR_$_CLASS_STRING_THROWABLE,
				new Object[] { this.testCase.getClass(), null, re2 })
				.ignoreArgument(1).setNumberInvocations(3);

		// Exercise.
		this.testCase.run(this.mockResult);

		// Verify.
		this.hammock.verify();
	}

	public void runTest() throws Throwable {
		if (getName().equals("testCountTestCases")) {
			testCountTestCases();
		} else if (getName().equals("testRunOk")) {
			testRunOk();
		} else if (getName().equals("testSetupException")) {
			testSetupException();
		} else if (getName().equals("testTearDownException")) {
			testTearDownException();
		} else if (getName().equals("testSetupTearDownException")) {
			testSetupTearDownExceptions();
		} else {
			fail("Unknown test: " + getName());
		}
	}

	public Test suite() {
		TestSuite suite = new TestSuite(this.getClass(), new String[] {
				"testCountTestCases", "testRunOk", "testSetupException",
				"testTearDownException", "testSetupTearDownException" });
		return suite;
	}
}
