package jmunit.framework.cldc11;

import j2meunit.framework.Test;
import j2meunit.framework.TestCase;
import j2meunit.framework.TestSuite;

public class MemoryMeasurementTest extends TestCase {

	// The OUT.
	private MemoryMeasurement memoryMeasurement;

	public void setUp() {
		this.memoryMeasurement = new MemoryMeasurement(1024);
	}

	public void testMemoryOk() {
		this.memoryMeasurement.startMeasurement();
		this.memoryMeasurement.endMeasurement();
	}

	public void testMemoryLeak() {
		this.memoryMeasurement.startMeasurement();
		byte[] b = new byte[2048];
		try {
			this.memoryMeasurement.endMeasurement();
			fail("More than 1 kB of memory leaked, but wasn't detected.");
		} catch (AssertionFailedException e) {

		}
		// Keep the IDE happy that b is used; prevents a warning.
		b[0] = 0;
		b = null;
	}

	public void testConstructorStartsMeasurement() {
		byte[] b = new byte[2048];
		try {
			this.memoryMeasurement.endMeasurement();
			fail("More than 1 kB of memory leaked, but wasn't detected.");
		} catch (AssertionFailedException e) {

		}
		// Keep the IDE happy that b is used; prevents a warning.
		b[0] = 0;
		b = null;
	}
	
	public void testGarbageCollector() {
		// Test without the GC.
		this.memoryMeasurement = new MemoryMeasurement(1024, false);
		byte[] b = new byte[2048];
		b[0] = 0;
		b = null;
		try {
			this.memoryMeasurement.endMeasurement();
			fail("More than 1 kB of memory leaked, but wasn't detected.");
		} catch (AssertionFailedException e) {

		}
		
		// Test with the GC.
		System.gc();
		this.memoryMeasurement = new MemoryMeasurement(1024, true);
		b = new byte[2048];
		b[0] = 0;
		b = null;
		this.memoryMeasurement.endMeasurement();
		
		// Test default, runs GC.
		System.gc();
		this.memoryMeasurement = new MemoryMeasurement(1024);
		b = new byte[2048];
		b[0] = 0;
		b = null;
		this.memoryMeasurement.endMeasurement();
	}

	public void runTest() throws Throwable {
		if (getName().equals("testMemoryOk")) {
			testMemoryOk();
		} else if (getName().equals("testMemoryLeak")) {
			testMemoryLeak();
		} else if (getName().equals("testConstructorStartsMeasurement")) {
			testConstructorStartsMeasurement();
		} else if (getName().equals("testGarbageCollector")) {
			testGarbageCollector();
		} else {
			fail("Unknown test: " + getName());
		}
	}

	public Test suite() {
		TestSuite suite = new TestSuite(this.getClass(), new String[] {
				"testMemoryOk", "testMemoryLeak",
				"testConstructorStartsMeasurement", "testGarbageCollector" });
		return suite;
	}
}
