package jmunit.framework.cldc11;

import j2meunit.framework.Test;
import j2meunit.framework.TestCase;
import j2meunit.framework.TestSuite;
import jmunit.framework.cldc11.mocks.MockPerformanceMeasurement;
import jmunit.framework.cldc11.mocks.MockTestCase;
import jmunit.framework.cldc11.mocks.MockTestResult;

import com.hammingweight.hammock.CompositeHandler;
import com.hammingweight.hammock.Hammock;

public class TestCasePerfTest extends TestCase {

	// The OUT.
	private MockTestCase testCasePerf = AllTestsRunner.testCase;

	// The mocks.
	private Hammock hammock;
	private MockTestResult mockResult;
	private MockPerformanceMeasurement mockPerfMeasurement;

	public void setUp() {
		this.hammock = new Hammock();
		this.mockResult = new MockTestResult(this.hammock);
		this.mockPerfMeasurement = new MockPerformanceMeasurement(this.hammock);
		this.testCasePerf.setInvocationHandler(new CompositeHandler());
		this.testCasePerf.addPerformanceMeasurement(this.mockPerfMeasurement);
		this.testCasePerf.setInvocationHandler(this.hammock);
	}

	public void tearDown() {
		this.testCasePerf.setInvocationHandler(new CompositeHandler());
		this.testCasePerf.removeAllPerformanceMeasurements();
	}

	public void testPerformance() {
		// Expectations.
		// Three tests should be started and stopped (twice)
		this.hammock.setExpectation(
				MockTestResult.MTHD_START_TEST_$_CLASS_STRING)
				.setNumberInvocations(6);
		this.hammock.setExpectation(MockTestCase.MTHD_SET_UP)
				.setNumberInvocations(6);
		this.hammock.setExpectation(MockTestCase.MTHD_TEST_$_INT)
				.setNumberInvocations(6);
		this.hammock.setExpectation(MockTestCase.MTHD_TEAR_DOWN)
				.setNumberInvocations(6);
		this.hammock
				.setExpectation(MockTestResult.MTHD_END_TEST_$_CLASS_STRING)
				.setNumberInvocations(6);
		// The performance measurement methods should be invoked three times.
		for (int i = 0; i < 3; i++) {
			this.hammock
					.setExpectation(MockPerformanceMeasurement.MTHD_START_MEASUREMENT);
			this.hammock
					.setExpectation(MockPerformanceMeasurement.MTHD_END_MEASUREMENT);
		}

		// Exercise.
		this.testCasePerf.run(this.mockResult);
		this.testCasePerf.setInvocationHandler(new CompositeHandler());
		this.testCasePerf
				.removePerformanceMeasurement(this.mockPerfMeasurement);
		this.testCasePerf.setInvocationHandler(this.hammock);
		this.testCasePerf.run(this.mockResult);

		// Verify.
		this.hammock.verify();
	}

	public void testRemoveAll() {
		this.testCasePerf.setInvocationHandler(new CompositeHandler());
		this.testCasePerf
				.addPerformanceMeasurement(new MockPerformanceMeasurement(
						this.hammock));
		this.testCasePerf
				.addPerformanceMeasurement(new MockPerformanceMeasurement(
						this.hammock));
		this.testCasePerf.removeAllPerformanceMeasurements();

		// Expectations.
		// Three tests should be started and stopped.
		this.hammock.setExpectation(
				MockTestResult.MTHD_START_TEST_$_CLASS_STRING)
				.setNumberInvocations(3);
		this.hammock.setExpectation(MockTestCase.MTHD_SET_UP)
				.setNumberInvocations(3);
		this.hammock.setExpectation(MockTestCase.MTHD_TEST_$_INT)
				.setNumberInvocations(3);
		this.hammock.setExpectation(MockTestCase.MTHD_TEAR_DOWN)
				.setNumberInvocations(3);
		this.hammock
				.setExpectation(MockTestResult.MTHD_END_TEST_$_CLASS_STRING)
				.setNumberInvocations(3);

		// Exercise.
		this.testCasePerf.setInvocationHandler(this.hammock);
		this.testCasePerf.run(this.mockResult);

		// Verify.
		this.hammock.verify();
	}

	// The test case should only respond to the first testEndMeasurement call.
	public void testEndMeasurement() {
		this.testCasePerf.setInvocationHandler(new CompositeHandler());

		// Expectations.
		this.hammock.setExpectation(
				MockPerformanceMeasurement.MTHD_START_MEASUREMENT)
				.setNumberInvocations(2);
		this.hammock.setExpectation(
				MockPerformanceMeasurement.MTHD_END_MEASUREMENT)
				.setNumberInvocations(2);

		// Exercise.
		this.testCasePerf.startPerformanceMeasurements();
		this.testCasePerf.endPerformanceMeasurements();
		this.testCasePerf.endPerformanceMeasurements(); // Should be ignored.
		this.testCasePerf.startPerformanceMeasurements();
		this.testCasePerf.endPerformanceMeasurements(); // Should be responded
														// to.

		// Verify.
		this.hammock.verify();
	}

	public void runTest() throws Throwable {
		if (getName().equals("testPerformance")) {
			testPerformance();
		} else if (getName().equals("testRemoveAll")) {
			testRemoveAll();
		} else if (getName().equals("testEndMeasurement")) {
			testEndMeasurement();
		} else {
			fail("Unknown test: " + getName());
		}
	}

	public Test suite() {
		TestSuite suite = new TestSuite(this.getClass(), new String[] {
				"testPerformance", "testRemoveAll", "testEndMeasurement" });
		return suite;
	}

}
